##########
# K8S-BASE 
##########

# prepare our configs
locals {
  secrets = flatten([for n, d in var.settings : [

    for s, dat in lookup(d, "secrets", {}) : merge({
      secret_name = s
      namespace   = n
      secret_type = "Opaque"
      data        = null
    }, dat)
  ]])

  secrets_map = { for item in local.secrets : item.secret_name => item }


  configmaps = flatten([for n, d in var.settings : [
    for c, dat in lookup(d, "configmaps", {}) : merge({
      config_name = c
      namespace   = n
      data        = null
    }, dat)
  ]])

  configs_map = { for item in local.configmaps : item.config_name => item }
}

##########
# HELM APPS
##########
locals {

 # current_apps = var.istio_enable ? var
  apps_defaults = {
    cert-manager = {
      namespace   = "cert-manager"
      chart       = "cert-manager"
      repository  = "https://charts.jetstack.io"
      version     = "1.2.0"
      max_history = 3,
      values = {
        file = [],
        set = {
          "installCRDs" = "true"
        }
      }
    },

    prometheus = {
      namespace   = "monitoring"
      chart       = "prometheus"
      repository  = "https://prometheus-community.github.io/helm-charts"
      version     = "13.3.2"
      max_history = 3,
      values = {
        file = [],
        set = {
          "alertmanager.enabled"                 = "false",
          "pushgateway.enabled"                  = "false",
          "server.persistentVolume.size"         = "20Gi",
          "server.persistentVolume.storageClass" = "",
          "alertmanagerFiles.alertmanager.yml"   = ""
        }
      }
    },

    grafana = {
      namespace   = "monitoring"
      chart       = "grafana"
      repository  = "https://grafana.github.io/helm-charts"
      version     = "6.4.2"
      max_history = 3,
      values = {
        file = [],
        set = {
          "persistence.enabled"          = "true",
          "persistence.storageClassName" = "",
        }
      }
    },

    ingress-nginx = {
      namespace   = "ingress-nginx"
      chart       = "ingress-nginx"
      repository  = "https://kubernetes.github.io/ingress-nginx"
      version     = "3.23.0"
      max_history = 3,
      values = {
        file = [
          file("${path.module}/values/ingress-nginx.yaml")
        ],
        set = {}
      }
    }

    metrics-server = {
      namespace   = "kube-system"
      chart       = "metrics-server"
      repository  = "https://charts.bitnami.com/bitnami"
      version     = "5.5.1"
      max_history = 3,
      values = {
        file = [],
        set = {
          "rbac.create"       = "true"
          "apiService.create" = "true"
        }
      }
    }

    datadog = {
      namespace   = "monitoring"
      chart       = "datadog"
      repository  = "https://helm.datadoghq.com"
      version     = "2.10.8"
      max_history = 3,
      force       = true,
      values = {
        file = [],
        set  = {}
      }
    }

    cluster-autoscaler = {
      namespace   = "kube-system"
      chart       = "cluster-autoscaler"
      repository  = "https://kubernetes.github.io/autoscaler"
      version     = "9.4.0"
      max_history = 3,
      values = {
        file = [],
        set = {
          "cloudProvider" = "aws"
        }
      }
    }
  }

  # extend the default settings with provided values
  expanded_apps_settings = { for k, v in var.apps :
    k => merge(v, length(lookup(local.apps_defaults, k, {})) == 0 ? tomap() : {
      namespace   = lookup(v, "namespace", local.apps_defaults[k]["namespace"]),
      chart       = lookup(v, "chart", local.apps_defaults[k]["chart"]),
      repository  = lookup(v, "repository", local.apps_defaults[k]["repository"]),
      version     = lookup(v, "version", local.apps_defaults[k]["version"]),
      max_history = lookup(v, "max_history", local.apps_defaults[k]["max_history"]),

      values = {
        file = concat(local.apps_defaults[k]["values"]["file"], lookup(lookup(v, "values", {}), "file", [])),
        set  = merge(local.apps_defaults[k]["values"]["set"], lookup(lookup(v, "values", {}), "set", {}))
      }
    })
  }
}

##########
# K8S-BASE INGRESS
##########
locals {
  ingresses = { for k, v in var.ingresses : v["name"] => v }
}